# coding: utf-8
"""
HCE project,  Python bindings, Distributed Tasks Manager application.
LogSwitch class content main functional of support 'LOGGER' property.

@package: app
@file LogSwitch.py
@author bgv bgv.hce@gmail.com, Alexander Vybornyh <alexander.hce.cluster@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2018 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""

# import time
import app.Utils as Utils


class LogSwitch(object):
  # # Constants used in class
  PARAMETER_SUFFIX = 'suffix'

  MACRO_PROJECT_ID = '%PROJECT_ID%'

  # # Constants of error messages
  ERROR_BAD_INPUT_DATA_TYPE = "Input data '%' has wrong type: %s"
  ERROR_EMPTY_INPUT_DATA = "Input data '%' is empty."
  ERROR_LOAD_PROPERTY = "Load property failed."

#   ## Initialization
#   #
#   # @param log - logger instance
#   # @param projectId - project ID
#   # @param propertyStr - property string
#   def __init__(self, log, projectId=None, propertyStr=None):
#     self.mpLogger = Utils.MPLogger()
#     self.logger = log
#     self.projectId = projectId
#
#     if self.projectId is not None and propertyStr is not None:
#       self.switchLoggerName(self.projectId, propertyStr)
#
#
#   ## Delete instance
#   def __del__(self):
#     if self.projectId is not None:
#       self.restoreLoggerName(self.projectId)


  ## Initialization
  def __init__(self):
    pass

#   ## load property
#   #
#   # @param properties - property string
#   # @param log - logger instance
#   # @return loaded property dictionary or None
#   @staticmethod
#   def loadProperty(properties, log=None):
#     # variable for result
#     ret = None
#     try:
#       if properties is None or properties == "":
#         raise Exception("Empty properties value.")
#
#       ret = Utils.jsonLoadsSafe(jsonString=properties, default=properties, log=log)
#       if ret is None:
#         raise Exception("Load properties failed.")
#
#     except Exception, err:
#       if log is not None:
#         log.error(str(err))
#
#     return ret

#   ## switch logger name
#   #
#   # @param projectId - project ID
#   # @param propertyStr - property string
#   # @return - None
#   def switchLoggerName(self, projectId, propertyStr):
#     self.logger.info("Switch logger to dedicated project %s", str(projectId))
#     self.setLogConfigFileProject(self.mpLogger, projectId, propertyStr, self.logger)
#
#
#   ## restore logger name
#   #
#   # @param projectId - project ID
#   # @return - None
#   def restoreLoggerName(self, projectId):
#     self.logger.info("Switch logger back to default from dedicated for project %s", str(projectId))
#     self.logger = self.setLogConfigFileDefault(self.mpLogger, self.logger)
#     self.logger.info("Switched logger back to default from dedicated for project %s", str(projectId))


  # #load project/site-specific logging
  # load logging configuration (log file, log level, filters) for project-specific file name
  #
  # @param mpLogger - multi process logger
  # @param projectId - project ID
  # @param propertyStr - property string
  # @param log - logger instance
  # @return logger instance
  @staticmethod
  def setLogConfigFileProject(mpLogger, projectId, propertyStr, log=None):  # pylint: disable=W0622
    try:
      if mpLogger is None:
        raise Exception(LogSwitch.ERROR_BAD_INPUT_DATA_TYPE % ('mpLogger', str(mpLogger)))

      if not isinstance(projectId, basestring):
        raise Exception(LogSwitch.ERROR_BAD_INPUT_DATA_TYPE % ('projectId', str(projectId)))

      if projectId=="":
        raise Exception(LogSwitch.ERROR_EMPTY_INPUT_DATA % ('projectId'))

      propertyDict = Utils.jsonLoadsSafe(jsonString=propertyStr, default=propertyStr, log=log)
      if not isinstance(propertyDict, dict):
        raise Exception(LogSwitch.ERROR_LOAD_PROPERTY)

      if LogSwitch.PARAMETER_SUFFIX in propertyDict:
        suffix = propertyDict[LogSwitch.PARAMETER_SUFFIX].replace(LogSwitch.MACRO_PROJECT_ID, projectId)
        log = mpLogger.getLogger(fileNameSuffix=suffix)

        log.info("===== Set project-specific logger =====")
        log.debug("Switched logger to dedicated project %s", str(projectId))
      else:
        if log is not None:
          log.warning("Suffix field not found for project %s in property: %s", str(projectId), str(propertyStr))

    except Exception, err:
      if log is not None:
        log.error("Error set project-specific logger: %s", str(err))

    return log


  # # set default logging
  #
  # @param mpLogger - multiple process logger
  # @param projectId - project ID
  # @param log - logger instance
  # @return restored logger instance
  @staticmethod
  def setLogConfigFileDefault(mpLogger, projectId, log=None):
    try:
      if mpLogger is None:
        raise Exception(LogSwitch.ERROR_BAD_INPUT_DATA_TYPE % ('mpLogger', str(mpLogger)))

      if not isinstance(projectId, basestring):
        raise Exception(LogSwitch.ERROR_BAD_INPUT_DATA_TYPE % ('projectId', str(projectId)))

      if projectId == "":
        raise Exception(LogSwitch.ERROR_EMPTY_INPUT_DATA % ('projectId'))

      if log is not None:
        log.info("Switch logger back to default from dedicated for project %s", str(projectId))

      log = mpLogger.getLogger(restore=True)
      if log is not None:
        log.info("Switched logger back to default from dedicated for project %s", str(projectId))

    except Exception, err:
      if log is not None:
        log.error("Error set default logger: %s", str(err))

    return log
