# coding: utf-8
"""
HCE project,  Python bindings, Distributed Tasks Manager application.
PDateFormat class content main functional of support 'PDATE_FORMAT' property.

@package: app
@file PDateFormat.py
@author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2018 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""

import re
import copy
import json
import datetime

import app.Utils as Utils
from app.DateTimeType import DateTimeType
from app.Exceptions import DatabaseException
from app.Utils import ExceptionLog
import app.Consts as APP_CONSTS


class PDateFormat(object):
  # # Constants used in class
  PROPERTY_PATTERN_NAME = 'pattern'
  PROPERTY_FORMAT_NAME = 'format'
  PROPERTY_MACRO_NAME = 'macro'
  PROPERTY_IF_NOT_DETECTED_NAME = 'if_not_detected'
  PROPERTY_STAGE = 'sources_mask'

  MACRO_NAME_MACRO = '%macro%'

  IF_NOT_DETECTED_VALUE_STOP = 0
  IF_NOT_DETECTED_VALUE_CONTINUE = 1
  DEFAULT_VALUE_IF_NOT_DETECTED = IF_NOT_DETECTED_VALUE_CONTINUE

  STAGE_RSS = APP_CONSTS.PDATE_SOURCES_MASK_RSS_FEED
  STAGE_HTTP_DATE = APP_CONSTS.PDATE_SOURCES_MASK_HTTP_DATE
  STAGE_HTTP_LAST_MODIFIED = APP_CONSTS.PDATE_SOURCES_MASK_HTTP_LAST_MODIFIED
  STAGE_DC_DATE = APP_CONSTS.PDATE_SOURCES_MASK_DC_DATE
  STAGE_PUBDATE = APP_CONSTS.PDATE_SOURCES_MASK_PUBDATE

  DEFAULT_VALUE_STAGE = STAGE_RSS + STAGE_PUBDATE


  # Initialization class
  #
  # @param property - property string
  # @param url - url string
  # @param log - logger instance
  def __init__(self, property, url, log=None):
    self.formatList, self.ifNotDetected, self.stage = self.loadProperty(property, url, log)


  # # load property
  #
  # @param property - property string
  # @param url - url string
  # @param log - logger instance
  # @return format list and flag ifNotDetected
  @staticmethod
  def loadProperty(property, url=None, log=None):
    # variables for result
    formatList = []
    ifNotDetected = PDateFormat.DEFAULT_VALUE_IF_NOT_DETECTED
    stage = PDateFormat.DEFAULT_VALUE_STAGE
    try:
      if isinstance(property, basestring) and property != "" and \
        property[0] != '{' and property[-1] != '}' and property[0] != '[' and property[-1] != ']':
        formatList.append(property)
      else:
        propertyObj = Utils.jsonLoadsSafe(jsonString=property, default=property, log=log)

#         if log is not None:
#           log.info("!!! propertyObj: %s", str(propertyObj))

        if isinstance(propertyObj, dict):
          formatList, ifNotDetected, stage = PDateFormat.extractValues(propertyDict=propertyObj, url=url, log=log)
        elif isinstance(propertyObj, list):
          for propertyDict in propertyObj:
            formatList, ifNotDetected, stage = PDateFormat.extractValues(propertyDict=propertyDict, url=url, log=log)
            if isinstance(formatList, list) and len(formatList) > 0:
              break

    except Exception, err:
      if log is not None:
        log.error(str(err))

    return formatList, ifNotDetected, stage


  # # extract values
  #
  # @param propertyDict - property dictionary
  # @param url - url string
  # @param log - logger instance
  # @return format list and flag ifNotDetected
  @staticmethod
  def extractValues(propertyDict, url=None, log=None):
    # variables for result
    formatList = []
    ifNotDetected = PDateFormat.DEFAULT_VALUE_IF_NOT_DETECTED
    stage = PDateFormat.DEFAULT_VALUE_STAGE
    try:
      if isinstance(propertyDict, dict):
        patternValue = propertyDict[PDateFormat.PROPERTY_PATTERN_NAME] if PDateFormat.PROPERTY_PATTERN_NAME in propertyDict else None
        formatValue = propertyDict[PDateFormat.PROPERTY_FORMAT_NAME] if PDateFormat.PROPERTY_FORMAT_NAME in propertyDict else None
        macroValue = propertyDict[PDateFormat.PROPERTY_MACRO_NAME] if PDateFormat.PROPERTY_MACRO_NAME in propertyDict else None
        ifNotDetected = int(propertyDict[PDateFormat.PROPERTY_IF_NOT_DETECTED_NAME]) if PDateFormat.PROPERTY_IF_NOT_DETECTED_NAME in propertyDict else \
          PDateFormat.DEFAULT_VALUE_IF_NOT_DETECTED
        stage = int(propertyDict[PDateFormat.PROPERTY_STAGE]) if PDateFormat.PROPERTY_STAGE in propertyDict else PDateFormat.DEFAULT_VALUE_STAGE

#         if log is not None:
#           log.info("!!! url: %s", str(url))

        if isinstance(formatValue, basestring) and isinstance(macroValue, list):
          if (isinstance(patternValue, basestring) and isinstance(url, basestring) and \
            re.search(pattern=patternValue, string=url, flags=re.U + re.I) is not None) or url is None:
            for macro in macroValue:
              formatString = copy.deepcopy(formatValue)
              formatList.append(re.sub(pattern=PDateFormat.MACRO_NAME_MACRO, repl=macro, string=formatString, flags=re.U + re.I))

    except Exception, err:
      if log is not None:
        log.error(str(err))

    return formatList, ifNotDetected, stage

