# coding: utf-8
"""
HCE project,  Python bindings, Distributed Tasks Manager application.
PDateSourceMask class content main functional of support 'PDATE_SOURCES_MASK' and 'PDATE_SOURCES_MASK_OVERWRITE' properties.

@package: app
@file PDateSourceMask.py
@author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2018 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""

import re

import app.Utils as Utils


class PDateSourceMask(object):
  # # Constants used in class
  PROPERTY_PATTERN_NAME = 'pattern'
  PROPERTY_MASK_NAME = 'mask'
  PROPERTY_DEFAULT_MASK_NAME = 'default_mask'
  PROPERTY_OVERWRITE_PATTERN_NAME = 'overwrite_pattern'
  PROPERTY_OVERWRITE_MASK_NAME = 'overwrite_mask'
  PROPERTY_OVERWRITE_DEFAULT_MASK_NAME = 'overwrite_default_mask'
  PROPERTY_FORMATS = 'formats'


  # Initialization class
  #
  # @param properties - properties JSON string or numeric
  # @param url - URL string
  # @param log - logger instance
  def __init__(self, properties, url, log=None):
    self.pdateSourceMask = None
    self.pdateSourceMaskOverwrite = None
    self.formats = None

    self.pdateSourceMask, self.pdateSourceMaskOverwrite, self.formats = PDateSourceMask.loadProperty(properties, url, log)


  ## load property
  #
  # @param properties - properties JSON string or numeric
  # @param url - URL string
  # @param log - logger instance
  # @return pdateSourceMask and pdateSourceMaskOverwrite values if can extracted or None values
  @staticmethod
  def loadProperty(properties, url, log=None):
    # variables for result
    pdateSourceMask = None
    pdateSourceMaskOverwrite = None
    formats = None
    try:
      if isinstance(properties, int):
        pdateSourceMask = properties
      elif isinstance(properties, basestring) and properties.isdigit():
        pdateSourceMask = int(properties)
      else:
        propertyObj = Utils.jsonLoadsSafe(jsonString=properties, default=property, log=log)

        if isinstance(propertyObj, dict):
          pdateSourceMask, pdateSourceMaskOverwrite, formats = PDateSourceMask.extractValues(propertyDict=propertyObj, url=url, log=log)
        elif isinstance(propertyObj, list):
          for propertyDict in propertyObj:
#             if log is not None:
#               log.debug("propertyDict: %s", str(propertyDict))

            pdateSourceMask, pdateSourceMaskOverwrite, formats = PDateSourceMask.extractValues(propertyDict=propertyDict, url=url, log=log)
            if pdateSourceMask is not None or pdateSourceMaskOverwrite is not None:
              break

    except Exception, err:
      if log is not None:
        log.error(str(err))

    return pdateSourceMask, pdateSourceMaskOverwrite, formats


  # # extract values
  #
  # @param propertyDict - property dictionary
  # @param url - url string
  # @param log - logger instance
  # @return pdateSourceMask and pdateSourceMaskOverwrite values if can extracted or None values
  @staticmethod
  def extractValues(propertyDict, url, log=None):
    # variables for result
    pdateSourceMask, formats = PDateSourceMask.extractValue(patternName=PDateSourceMask.PROPERTY_PATTERN_NAME,
                                                   maskName=PDateSourceMask.PROPERTY_MASK_NAME,
                                                   defaultMaskName=PDateSourceMask.PROPERTY_DEFAULT_MASK_NAME,
                                                   propertyDict=propertyDict,
                                                   url=url,
                                                   log=log)

    if log is not None:
      log.debug("pdateSourceMask: %s, formats: %s", str(pdateSourceMask), str(formats))

    pdateSourceMaskOverwrite, notUsedFormats = PDateSourceMask.extractValue(patternName=PDateSourceMask.PROPERTY_OVERWRITE_PATTERN_NAME,
                                                   maskName=PDateSourceMask.PROPERTY_OVERWRITE_MASK_NAME,
                                                   defaultMaskName=PDateSourceMask.PROPERTY_OVERWRITE_DEFAULT_MASK_NAME,
                                                   propertyDict=propertyDict,
                                                   url=url,
                                                   log=log)

    if log is not None:
      log.debug("pdateSourceMaskOverwrite: %s, notUsedFormats: %s", str(pdateSourceMaskOverwrite), str(notUsedFormats))

    return pdateSourceMask, pdateSourceMaskOverwrite, formats


  # # extract value
  #
  # @param patternName - pattern name
  # @param maskName - mask name
  # @param defaultMaskName - default mask name
  # @param propertyDict - property dictionary
  # @param url - url string
  # @param log - logger instance
  # @return pdateSourceMask value if can extracted or None
  @staticmethod
  def extractValue(patternName, maskName, defaultMaskName, propertyDict, url, log=None):
    # variables for result
    ret = None
    formats = None
    try:
      if isinstance(propertyDict, dict) and isinstance(url, basestring):

        pattern = propertyDict[patternName] if patternName in propertyDict else None

#         if log is not None:
#           log.debug("pattern: %s", str(pattern))

        if isinstance(pattern, basestring) and re.search(pattern=pattern, string=url, flags=re.U + re.I) is not None:
          mask = int(propertyDict[maskName]) if maskName in propertyDict else None
          if isinstance(mask, int):
            ret = mask
          else:
            defaultMaskName = int(propertyDict[defaultMaskName]) if defaultMaskName in propertyDict else None
            if isinstance(defaultMaskName, int):
              ret = defaultMaskName

          formats = propertyDict[PDateSourceMask.PROPERTY_FORMATS] if PDateSourceMask.PROPERTY_FORMATS in propertyDict else None

#       if log is not None:
#         log.debug("ret = %s, formats: %s", str(ret), str(formats))

    except Exception, err:
      if log is not None:
        log.error(str(err))

    return ret, formats
