#coding: utf-8
'''
HCE project, Python bindings, requests cashe.


@package: dcc
@author bgv bgv.hce@gmail.com
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2018 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
'''

try:
  import cPickle as pickle
except ImportError:
  import pickle
import os
import time
from hashlib import md5
from hashlib import sha1

import app.Utils as Utils


# #The DCC helper class
#
#
class RequestsCashe(object):

  MODE_OFF = 0
  MODE_ON = 1


  # initialize fields
  #@param logger instance of logger
  #@param mode - 0 is OFF, 1 - is ON full functional, 2 - is ON write only
  #@param ttl - time to leave ms, defines a time for a response in cache treated as valid
  #@param sharding - options dict, if None - set by default:
  #                  {u'file_name_chars_from_start':2, u'id_type':0, u'cashe_dir':u'/tmp/dc_client_requests_cache', u'serialize':0}
  #                  u'id_type' - 0 is md5 of request serialized, serialize - 0 means pickle, 1 - means json
  def __init__(self, logger=None, mode=0, ttl=0, sharding=None):
    self.logger = logger
    self.mode = mode
    self.ttl = ttl
    if sharding is None:
      self.sharding = {u'file_name_chars_from_start':2, u'id_type':0, u'cashe_dir':u'/tmp/dc_client_requests_cache/', u'serialize':0, u'file_mask':u'666',
                       u'dir_mask':u'777'}
    else:
      self.sharding = sharding


  def get(self, uid, default=None):
    ret = default

    if self.mode == self.MODE_ON:
      try:
        path = self.getPath(uid)
        fn = self.getFileName(path, uid)
        if os.path.isfile(fn):
          t = os.stat(fn).st_mtime
          if (time.time() - t) < self.ttl:
            with open(fn, u'rb') as f:
              ret = f.read()
              if self.logger is not None:
                self.logger.debug("Read from the cache file: `%s`, %s bytes", fn, str(len(ret)))
              ret = self.deserialize(ret)
      except Exception, err:
        if self.logger is not None:
          self.logger.error("Error get from cache, uid: `%s` : %s\n%s", str(uid), str(err), Utils.getTracebackInfo())

    return ret


  def set(self, uid, dataOject):
    if self.mode == self.MODE_ON:
      try:
        serializedObject = self.serialize(dataOject)
        path = self.getPath(uid)
        os.system(u'mkdir -p ' + path)
        if u'dir_mask' in self.sharding and self.sharding[u'dir_mask'] != u'':
          os.system(u'chmod -f ' + self.sharding[u'dir_mask'] + u' ' + path)
        fn = self.getFileName(path, uid)
        if os.path.isfile(fn):
          os.system(u'rm -f ' + fn)
        with open(fn, u'wb') as f:
          f.write(serializedObject)
        if u'file_mask' in self.sharding and self.sharding[u'file_mask'] != u'':
          os.system(u'chmod -f ' + self.sharding[u'file_mask'] + u' ' + fn)
        if self.logger is not None:
          self.logger.debug("Written to the cache file: `%s`, %s bytes", fn, str(len(serializedObject)))
      except Exception, err:
        if self.logger is not None:
          self.logger.error("Error set data to cache, uid: `%s` : %s\n%s", str(uid), str(err), Utils.getTracebackInfo())


  #@param data - data
  #
  #@return a serialized object
  def serialize(self, data):
    #if self.sharding[u'serialize'] == 0:
    serializedObject = pickle.dumps(data)

    return serializedObject


  #@param data - data
  #
  #@return a sdeerialized object
  def deserialize(self, data):
    #if self.sharding[u'serialize'] == 0:
    deserializedObject = pickle.loads(data)

    return deserializedObject


  #@param data - data
  #@param uidType - 0 md5, 1 - sha1
  #
  #@return an id string
  def getUid(self, data, uidType=0):
    uid = None

    try:
      #if self.sharding[u'serialize'] == 0:
      #serializedObject = base64.encodestring(self.serialize(data))
      serializedObject = self.serialize(data)

      if uidType == 0:
        uid = md5(serializedObject).hexdigest()
      else:
        uid = sha1(serializedObject).hexdigest()
    except Exception, err:
      self.logger.error("Error get uid : %s\n%s", str(err), Utils.getTracebackInfo())

    return uid


  # Returns a cache path for uid
  #@param uid
  #
  #@return an id string
  def getPath(self, uid):
    dirLevel = uid[:2]
    ret = self.sharding[u'cashe_dir'] + dirLevel

    return ret


  # Returns a file name string
  #@param path
  #@param uid
  #
  #@return a file name string
  def getFileName(self, path, uid):
    return path + u'/' + uid
