# coding: utf-8
"""
HCE project,  Python bindings, Distributed Tasks Manager application.
UrlsLogging class content main functional of support 'URLS_LOGGING' property.

@package: app
@file UrlsLogging.py
@author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2018 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""

import os
import re
import datetime
import app.Utils as Utils
import dc_db.Constants as DB_CONSTANTS
import dc_crawler.Constants as CRAWLER_CONSTANTS
import dbi.EventObjects


# # The UrlsLogging class for support of a URLs lists logging
class UrlsLogging(object):
  # # Constants used in class
  PROPERTY_FILE_NAME = "file"
  PROPERTY_TIMESTAMP_NAME = "timestamp"
  PROPERTY_DELIMITER_NAME = "delimiter"
  PROPERTY_FIELDS_NAME = "fields"

  PROPERTY_STAGE_FEED_NAME = "feed"
  PROPERTY_STAGE_COLLECT_NAME = "to_collect"
  PROPERTY_STAGE_INSERT_NAME = "to_insert"
  PROPERTY_STAGE_SKIPPED_NAME = "skipped"
  PROPERTY_STAGE_AGED_NAME = "aged"
  PROPERTY_STAGE_FRESHEN_NAME = "freshen"
  PROPERTY_STAGE_DELETED_NAME = "deleted"

  # Support stage constants
  STAGE_FEED = 0
  STAGE_COLLECT = 1
  STAGE_INSERT = 2
  STAGE_SKIPPED = 3
  STAGE_AGED = 4
  STAGE_FRESHEN = 5
  STAGE_DELETED = 6

  STAGES_NAMES = {STAGE_FEED:PROPERTY_STAGE_FEED_NAME,
                  STAGE_COLLECT:PROPERTY_STAGE_COLLECT_NAME,
                  STAGE_INSERT:PROPERTY_STAGE_INSERT_NAME,
                  STAGE_SKIPPED:PROPERTY_STAGE_SKIPPED_NAME,
                  STAGE_AGED:PROPERTY_STAGE_AGED_NAME,
                  STAGE_FRESHEN:PROPERTY_STAGE_FRESHEN_NAME,
                  STAGE_DELETED:PROPERTY_STAGE_DELETED_NAME}

  DEFAULT_VALUE_TIMESTAMP = "%Y-%m-%d %H:%M:%S"
  DEFAULT_VALUE_DELIMITER = ","
  DEFAULT_VALUE_FIELDS = []

  # # Constants of macro used in class
  MACRO_PROJECT_ID = "%PROJECT_ID%"

  # # Constants of error messages
  ERROR_MSG_TYPE_INPUT_VALUE = "Wrong type (%s) of the input value for '%s'"
  ERROR_MSG_PROPERTY_TYPE = "Wrong type (%s) of the property: %s"
  ERROR_MSG_UNSUPPORTED_VALUE = "Unsupported value '%s' for '%s'"


  # # Initialization
  def __init__(self):
    pass


  # # Add URL to log
  #
  # @param properties - properties value
  # @param url - URL string
  # @param siteId - site ID (project ID)
  # @param stage - stage for apply
  # @param urlObj - URL's instance
  # @param dbWrapper - DBTasksWrapper instance
  # @param log - logger instance
  # @return - None
  @staticmethod
  def addUrl(properties, url, siteId, stage, urlObj=None, dbWrapper=None, log=None):
    try:
      if properties is None:
        raise Exception(UrlsLogging.ERROR_MSG_TYPE_INPUT_VALUE % (str(type(properties)), 'properties'))

      if not isinstance(url, basestring):
        raise Exception(UrlsLogging.ERROR_MSG_TYPE_INPUT_VALUE % (str(type(url)), 'url'))

      if not isinstance(siteId, basestring):
        raise Exception(UrlsLogging.ERROR_MSG_TYPE_INPUT_VALUE % (str(type(siteId)), 'siteId'))

      if not isinstance(stage, int):
        raise Exception(UrlsLogging.ERROR_MSG_TYPE_INPUT_VALUE % (str(type(stage)), 'stage'))

      propertyDict = Utils.jsonLoadsSafe(jsonString=properties, default=property, log=None)
      if not isinstance(propertyDict, dict):
        raise Exception(UrlsLogging.ERROR_MSG_PROPERTY_TYPE % (str(type(propertyDict)), str(propertyDict)))

      if stage not in UrlsLogging.STAGES_NAMES:
        raise Exception(UrlsLogging.ERROR_MSG_UNSUPPORTED_VALUE % (str(stage), 'stage'))

      stageName = UrlsLogging.STAGES_NAMES[stage]
      if stageName in propertyDict:
        if not isinstance(propertyDict[stageName], dict):
          raise Exception(UrlsLogging.ERROR_MSG_TYPE_INPUT_VALUE % (str(type(propertyDict[stageName])), str(stageName)))

      fields = {}
      if UrlsLogging.PROPERTY_FIELDS_NAME in propertyDict[stageName] and isinstance(propertyDict[stageName][UrlsLogging.PROPERTY_FIELDS_NAME], list) and \
        len(propertyDict[stageName][UrlsLogging.PROPERTY_FIELDS_NAME]) > 0:
        fields = UrlsLogging.getFields(url=url, siteId=siteId, urlObj=urlObj, dbWrapper=dbWrapper, log=log)

      if log is not None:
        log.info("fields: %s", str(fields))

      UrlsLogging.writeData(stagePropertyDict=propertyDict[stageName], dataStr=url, siteId=siteId, fields=fields, log=log)

    except Exception, err:
      if log is not None:
        log.error(str(err))

  # # Get fields values
  #
  # @param url - URL string
  # @param siteId - site ID (project ID)
  # @param urlObj - URL's object instance
  # @param dbWrapper - DBTasksWrapper instance
  # @param log - logger instance
  @staticmethod
  def getFields(url, siteId, urlObj=None, dbWrapper=None, log=None):
    #variable for result
    fields = {}
    try:
      if urlObj is not None:
        for key, value in urlObj.__dict__.items():
          fields[key.lower()] = value

      if dbWrapper is not None:
        if not isinstance(url, basestring) or url == "":
          raise Exception(UrlsLogging.ERROR_MSG_UNSUPPORTED_VALUE % (str(url), 'url'))

        if not isinstance(siteId, basestring) or siteId == "":
          raise Exception(UrlsLogging.ERROR_MSG_UNSUPPORTED_VALUE % (str(siteId), 'siteId'))

        selectUrlsQuery = "SELECT * FROM %s WHERE `URL` = '%s' LIMIT 1" % (DB_CONSTANTS.DC_URLS_TABLE_NAME_TEMPLATE % str(siteId), str(url))
        if log is not None:
          log.debug("selectUrlsQuery: %s", str(selectUrlsQuery))

        affectDB = dbWrapper.affect_db
        dbWrapper.affect_db = True
        customResponse = dbWrapper.customRequest(selectUrlsQuery, CRAWLER_CONSTANTS.DB_URLS, includeFieldsNames=dbi.EventObjects.CustomRequest.SQL_BY_NAME)
        dbWrapper.affect_db = affectDB

        if log is not None:
          log.debug("customResponse: %s", str(customResponse))

        if customResponse is not None and len(customResponse) > 0:
          if log is not None:
            log.debug("type(customResponse[0]) =  %s", str(type(customResponse[0])))

          if isinstance(customResponse[0], dict):
            for key, value in customResponse[0].items():
              if log is not None:
                log.debug("key: '%s'", str(key))

              for urlTableDictKey, urlTableDictValue in DB_CONSTANTS.URLTableDict.items():
                if urlTableDictValue == key:
                  if log is not None:
                    log.debug("urlTableDictKey: '%s'", str(urlTableDictKey))

                  fields[urlTableDictKey.lower()] = value
                  break
        else:
          for urlTableDictKey in DB_CONSTANTS.URLTableDict.keys():
            fields[urlTableDictKey.lower()] = None

    except Exception, err:
      if log is not None:
        log.error(str(err))

    return fields


  # # write data to log
  #
  # @param stagePropertyDict - stage property dictionary
  # @param dataStr - data string
  # @param siteId - site ID (project ID)
  # @param fields - fields dictionary
  # @param log - logger instance
  # @return - None
  @staticmethod
  def writeData(stagePropertyDict, dataStr, siteId, fields=None, log=None):
    try:
      if not isinstance(stagePropertyDict, dict):
        raise Exception(UrlsLogging.ERROR_MSG_TYPE_INPUT_VALUE % (str(type(stagePropertyDict)), 'stagePropertyDict'))

      if not isinstance(dataStr, basestring) or dataStr == "":
        raise Exception(UrlsLogging.ERROR_MSG_UNSUPPORTED_VALUE % (str(dataStr), 'dataStr'))

      if not isinstance(siteId, basestring) or siteId == "":
        raise Exception(UrlsLogging.ERROR_MSG_UNSUPPORTED_VALUE % (str(siteId), 'siteId'))

      if UrlsLogging.PROPERTY_FILE_NAME in stagePropertyDict:
        fileName = stagePropertyDict[UrlsLogging.PROPERTY_FILE_NAME]
        fileName = re.sub(pattern=UrlsLogging.MACRO_PROJECT_ID, repl=siteId, string=fileName, flags=re.U + re.I)

        if not isinstance(fileName, basestring) or fileName == "":
          raise Exception(UrlsLogging.ERROR_MSG_UNSUPPORTED_VALUE % (str(fileName), UrlsLogging.PROPERTY_FILE_NAME))

        fieldsList = stagePropertyDict[UrlsLogging.PROPERTY_FIELDS_NAME] if UrlsLogging.PROPERTY_FIELDS_NAME in stagePropertyDict and \
                     isinstance(stagePropertyDict[UrlsLogging.PROPERTY_FIELDS_NAME], list)  else UrlsLogging.DEFAULT_VALUE_FIELDS

        timestampFormat = stagePropertyDict[UrlsLogging.PROPERTY_TIMESTAMP_NAME] if UrlsLogging.PROPERTY_TIMESTAMP_NAME in stagePropertyDict and \
                          isinstance(stagePropertyDict[UrlsLogging.PROPERTY_TIMESTAMP_NAME], basestring) and \
                          stagePropertyDict[UrlsLogging.PROPERTY_TIMESTAMP_NAME] != "" else UrlsLogging.DEFAULT_VALUE_TIMESTAMP

        delimiter = stagePropertyDict[UrlsLogging.PROPERTY_DELIMITER_NAME] if UrlsLogging.PROPERTY_DELIMITER_NAME in stagePropertyDict and \
                    isinstance(stagePropertyDict[UrlsLogging.PROPERTY_DELIMITER_NAME], basestring) and \
                    stagePropertyDict[UrlsLogging.PROPERTY_DELIMITER_NAME] != "" else UrlsLogging.DEFAULT_VALUE_DELIMITER

#         if log is not None:
#           log.debug("fileName: %s", str(fileName))
#           log.debug("fieldsList: %s", str(fieldsList))
#           log.debug("timestampFormat: %s", str(timestampFormat))
#           log.debug("delimiter: %s", str(delimiter))

        # make data string
        dataString = datetime.datetime.now().strftime(timestampFormat) + delimiter + dataStr
        if isinstance(fields, dict):
          for fieldName in fieldsList:
            if fieldName.lower() in fields:
              dataString += delimiter + str(fields[fieldName])

        # write to file
        f = open(fileName, 'a' if os.path.isfile(fileName) else 'w')
        f.write(dataString + '\n')
        f.close()

    except Exception, err:
      if log is not None:
        log.error(str(err))
