# coding: utf-8

"""
HCE project,  Python bindings, Distributed Tasks Manager application.
FetcherType Class content main functional extract of fetcher options from properties.

@package: dc_crawler
@file FetcherType.py
@author Aleksandr Skuridin <skuridin.alexander+hce@gmail.com>, Alexander Vybornyh <alexander.hce.cluster@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2018 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""


import re
import json

import app.Utils as Utils
from dc_crawler.Fetcher import BaseFetcher

class FetcherType(object):
  # # Constants used in class
  MSG_ERROR_WRONG_TYPE_INPUT_OPTIONS = "Wrong type of input options. Input type = %s"
  MSG_ERROR_WRONG_VALUE_ELEMENT = "Wrong value of element '%s'. Value: %s"
  MSG_ERROR_WRONG_TYPE_ELEMENT = "Wrong type of element '%s'. Input type = %s"


  # # get the fetcher type by FETCHER_TYPE project property (obsolete)
  #
  #@param propertyValue - json string with pattern rules
  #@param siteUrl - site URL string with pattern rules. Only for debug purpose
  #@param logger - instance of logger for log if necessary
  #@return extracted fetcher type
  @staticmethod
  def getFromProperty(propertyValue, siteUrl, logger=None):
    #isExtendLog = False
    if logger is not None:
      logger.debug(siteUrl + ', FETCHER_TYPE: ' + propertyValue)

    fetchType = None
    try:
      fetcherProperties = json.loads(propertyValue)
      for pattern in fetcherProperties:
        match = re.search(pattern, siteUrl)
        if match:
          fetchType = fetcherProperties[pattern]
          # if fetchType in BaseFetcher.fetchers:
          if fetchType == BaseFetcher.TYP_DYNAMIC or fetchType == BaseFetcher.TYP_NORMAL:
            if logger is not None:
              logger.info(siteUrl + ', Fetch Type value: ' + str(fetchType))
          else:
            logger.debug(siteUrl + ', wrong Fetch Type number: ' + str(fetchType))
            fetchType = None
            continue
          break
    except Exception, ex:
      if logger is not None:
        logger.debug("Fetcher Type Exception: " + str(ex))

    return fetchType


  # # Internal class
  class FetcherTypeOptions(object):
    # # Constants used in class
    PROPERTY_OPTION_FETCHER_TYPE = 'fetcher_type'
    PROPERTY_OPTION_FETCHER_OPTIONS = 'fetcher_options'


    # Initialization
    def __init__(self, type=None, options=None):
      self.type = type
      self.options = options


  # # Initialization
  #
  # @param propertyValue - json string with pattern rules
  # @param log - instance of logger if necessary
  def __init__(self, propertyValue, log=None):
    self.properties = Utils.jsonLoadsSafe(jsonString=propertyValue, default={}, log=log)
    self.logger = log


  # # Extract options
  #
  # @param properties - properties dictionary
  # @param url - url string with pattern rules.
  # @param log - instance of logger for log if necessary
  # @return found fetcher type options structure or None
  @staticmethod
  def extractOptions(properties, url, log=None):
    # variable for result
    ret = None
    try:
      if not isinstance(properties, dict):
        raise Exception(FetcherType.MSG_ERROR_WRONG_TYPE_INPUT_OPTIONS % str(type(properties)))

      for key, value in properties.items():
        try:
          if re.search(pattern=key, string=url, flags=re.I + re.U) is not None:
            fetcherTypeOptions = FetcherType.FetcherTypeOptions()

            # extract values
            if isinstance(value, dict):
              if FetcherType.FetcherTypeOptions.PROPERTY_OPTION_FETCHER_TYPE in value:
                fetcherTypeOptions.type = int(value[FetcherType.FetcherTypeOptions.PROPERTY_OPTION_FETCHER_TYPE])

              if FetcherType.FetcherTypeOptions.PROPERTY_OPTION_FETCHER_OPTIONS in value:
                fetcherTypeOptions.options = value[FetcherType.FetcherTypeOptions.PROPERTY_OPTION_FETCHER_OPTIONS]

            else:
              fetcherTypeOptions.type = int(value)

            # validate values
            if fetcherTypeOptions.type != BaseFetcher.TYP_DYNAMIC and fetcherTypeOptions.type != BaseFetcher.TYP_NORMAL:
              raise Exception(FetcherType.MSG_ERROR_WRONG_VALUE_ELEMENT % (FetcherType.FetcherTypeOptions.PROPERTY_OPTION_FETCHER_TYPE,
                                                                           str(fetcherTypeOptions.type)))

            if fetcherTypeOptions.options is not None and not isinstance(fetcherTypeOptions.options, dict):
              raise Exception(FetcherType.MSG_ERROR_WRONG_TYPE_ELEMENT % (FetcherType.FetcherTypeOptions.PROPERTY_OPTION_FETCHER_OPTIONS,
                                                                          str(type(fetcherTypeOptions.options))))

            ret = fetcherTypeOptions
            break
        except Exception, err:
          if log is not None:
            log.error(str(err))

    except Exception, err:
      if log is not None:
        log.error(str(err))

    return ret


  # # Get fetcher type options
  #
  # @param url - url string with pattern rules.
  # @return found fetcher type options structure
  def getFetcherTypeOptions(self, url):
    return self.extractOptions(self.properties, url, self.logger)
