# coding: utf-8
"""
HCE project,  Python bindings, Distributed Tasks Manager application.
HTMLCanonicalRedirect Class content main functional for resolve canonical redirects

@package: dc_crawler
@file HTMLCanonicalRedirect.py
@author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2018 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""

import re
import app.Utils as Utils


class HTMLCanonicalRedirect(object):
  ## Constant error messages used in class
  ERROR_MSG_UNSUPPORTED_STATE_VALUE = "Unsupported state (%s) for domain '%s'"
  ERROR_MSG_WRONG_TYPE = "Wrong type (%s) for '%s'"

  # # Constants used in class
  PROPERTY_DOMAIN_NAME = 'domain'
  PROPERTY_STATE_NAME = 'state'

  STATE_VALUE_OFF = 0
  STATE_VALUE_ON = 1

  STATE_SUPPORT_VALUES = [STATE_VALUE_OFF, STATE_VALUE_ON]

  REDIRECT_PATTERN = r'"canonical" href="(.*)">'


  # # Initialization
  def __init__(self):
    pass


  ## extract canonical redirect value
  #
  # @param rawContent - raw content buffer
  # @param properties - site properties
  # @param url - url string
  # @param log - logger instance
  # @return canonical redirect string value
  @staticmethod
  def extract(rawContent, properties, url, log=None):
    # variable for result
    ret = None
    try:
      if not isinstance(rawContent, basestring):
        raise Exception(HTMLCanonicalRedirect.ERROR_MSG_WRONG_TYPE % (str(type(rawContent)), 'rawContent'))

      if not isinstance(url, basestring):
        raise Exception(HTMLCanonicalRedirect.ERROR_MSG_WRONG_TYPE % (str(type(rawContent)), 'url'))

      propertyDict = Utils.jsonLoadsSafe(jsonString=properties, default=properties, log=log)

      propertyDictList = []
      if isinstance(propertyDict, dict):
        propertyDictList = [propertyDict]
      elif isinstance(propertyDict, list):
        propertyDictList = [ n  for n in propertyDict if isinstance(n, dict) ]

      for propertyObj in propertyDictList:
        state = HTMLCanonicalRedirect.getState(propertyObj, url, log)
        if state is not None and state == HTMLCanonicalRedirect.STATE_VALUE_ON:
          # check raw content
          canonicalRedirects = re.findall(pattern=HTMLCanonicalRedirect.REDIRECT_PATTERN, string=rawContent, flags=re.U + re.I + re.M)
          if log is not None:
            log.debug("Extracted canonical url: %s", str(canonicalRedirects))

          if len(canonicalRedirects) > 0:
            ret = canonicalRedirects[0]
            break

    except Exception, err:
      if log is not None:
        log.error(str(err))

    return ret


  ## get state value
  #
  # @param propertyDict - site property dictionary
  # @param url - url string
  # @param log - logger instance
  # @return state value if it was extracted or otherwise None
  @staticmethod
  def getState(propertyDict, url, log=None):
    #variable for result
    ret = None
    try:
      if HTMLCanonicalRedirect.PROPERTY_DOMAIN_NAME in propertyDict and \
        isinstance(propertyDict[HTMLCanonicalRedirect.PROPERTY_DOMAIN_NAME], basestring) and  propertyDict[HTMLCanonicalRedirect.PROPERTY_DOMAIN_NAME] != "":
        # check domain
        domain = propertyDict[HTMLCanonicalRedirect.PROPERTY_DOMAIN_NAME]
        if re.search(pattern=domain, string=url, flags=re.U + re.I) is not None:
          if log is not None:
            log.debug("Found rule for domain '%s'", domain)

          # extract state
          if HTMLCanonicalRedirect.PROPERTY_STATE_NAME in propertyDict:
            state = int(propertyDict[HTMLCanonicalRedirect.PROPERTY_STATE_NAME])
            # check state
            if state not in HTMLCanonicalRedirect.STATE_SUPPORT_VALUES:
              raise Exception(HTMLCanonicalRedirect.ERROR_MSG_UNSUPPORTED_STATE_VALUE % (str(state), domain))

            ret = state
            if log is not None:
              log.debug("Return state: %s", str(state))

    except Exception, err:
      if log is not None:
        log.error(str(err))

    return ret
