# coding: utf-8
"""
HCE project, Python bindings, Distributed Tasks Manager application.
HTMLRecover it's class content main functional for 'HTML_RECOVER' property.

@package: dc_crawler
@file HTMLRecover.py
@author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2018 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""

import tidylib
import lxml.html

import app.Consts as APP_CONSTS

class HTMLRecover(object):
  # # Constants used in class
  OPTION_VALUE_NEVER = 0
  OPTION_VALUE_ALWAYS = 1
  OPTION_VALUE_PARSER_FAILED = 2

  SUPPORT_VALUES = [OPTION_VALUE_NEVER, OPTION_VALUE_ALWAYS, OPTION_VALUE_PARSER_FAILED]

  DEFAULT_CHARSET = 'utf-8'

  # # Constants of error messages
  ERROR_INPUT_TYPE = "Wrong input type %s instead expected %s"
  ERROR_INPUT_VALUE = "Wrong input value %s. Support: %s"


  # # Initialization
  def __init__(self):
    pass


  # # get tidy content
  #
  # @param htmlContent - html content as string
  # @param charset - charset of the content
  # @param options - options for tidy document
  # @log - logger instance
  # @return tidy content string
  @staticmethod
  def getTidyContent(htmlContent, charset, options=None, log=None):
    # variable for result
    ret = htmlContent
    try:
      if log is not None:
        log.debug("Try to fix DOM by tidylib.")

      ret, errors = tidylib.tidy_document(htmlContent.decode(HTMLRecover.DEFAULT_CHARSET).encode(charset), options=options)
      if errors != "" and  log is not None:
        log.debug("tidylib errors: %s", str(errors))

    except Exception, err:
      if log is not None:
        log.warning(str(err))

    return ret


  # # execute HTML recover
  #
  # @param siteProperties - site properties
  # @param charset - charset of the content
  # @param parser - HTML parser instance
  # @param log - logger instance
  # @param parserFailed - boolean flag has True if parser failed
  # @param defaultValue - default value
  # @return tuple of recovered html content string and tidy content string
  @staticmethod
  def execute(siteProperties, htmlContent, charset, parser, log=None, parserFailed=True, defaultValue=None):
    # variable for result
    recoveredHtmlContent = defaultValue
    tidyContent = None
    try:
      if not isinstance(siteProperties, dict):
        raise Exception(HTMLRecover.ERROR_INPUT_TYPE % (str(type(siteProperties)), str(type(dict()))))

      if APP_CONSTS.HTML_RECOVER in siteProperties and siteProperties[APP_CONSTS.HTML_RECOVER] is not None:
        if log is not None:
          log.debug("Found property '%s': %s", str(APP_CONSTS.HTML_RECOVER), str(siteProperties[APP_CONSTS.HTML_RECOVER]))

        value = int(siteProperties[APP_CONSTS.HTML_RECOVER])
        if value not in HTMLRecover.SUPPORT_VALUES:
          raise Exception(HTMLRecover.ERROR_INPUT_VALUE % (str(value), str(HTMLRecover.SUPPORT_VALUES)))

        if value == HTMLRecover.OPTION_VALUE_ALWAYS or (value == HTMLRecover.OPTION_VALUE_PARSER_FAILED and parserFailed):
          tidyContent = HTMLRecover.getTidyContent(htmlContent=htmlContent, charset=charset, log=log)
          recoveredHtmlContent = lxml.html.fromstring(tidyContent, parser=parser)

    except Exception, err:
      if log is not None:
        log.error(str(err))

    return recoveredHtmlContent, tidyContent
