# coding: utf-8
"""
HCE project,  Python bindings, Distributed Tasks Manager application.
HTTPRedirectExtractor Class content main functional for 'HTTP_HEADERS' property.

@package: dc_crawler
@file HTTPHeadersExtractor.py
@author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2018 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""
import re
from random import randint

import app.Utils as Utils
import app.Consts as APP_CONSTS
from app.LFSDataStorage import LFSDataStorage


class HTTPHeadersExtractor(object):
  # # Constants used in class
  PROPERTY_DOMAIN_NAME = 'domain'
  PROPERTY_HEADERS_NAME = 'headers'
  PROPERTY_STORAGE_NAME = 'storage'

  FILE_SOURCE_PREFIX = 'file://'
  DISABLE_CHARACTER = '#'

  ALGORITHM_RANDOM = 0
  ALGORITHM_LOW_FREQUENCY = 1

  DEFAULT_ALGORITHM_VALUE = ALGORITHM_RANDOM
  DEFAULT_SITEID_VALUE = '0'

  ARGUMENT_ALGORITHM = 'algorithm'
  ARGUMENT_SITEID = 'siteId'
  ARGUMENT_DOMAIN = 'domain'
  ARGUMENT_STORAGE_DIR = 'storageDir'

  # # Initialization
  def __init__(self):
    pass


  # # get site headers
  #
  # @param properties - properties value
  # @param url - url string
  # @param log - logger instance
  # @param kwarg - named arguments
  # @return site headers dictionary
  @staticmethod
  def getSiteHeaders(properties, url=None, log=None, **kwarg):
    # variable for result
    siteHeaders = {}
    saved = False

    try:
      jsonString = properties
      if isinstance(properties, basestring) and properties.startswith(HTTPHeadersExtractor.FILE_SOURCE_PREFIX):
        jsonString = Utils.loadFromFileByReference(fileReference=properties, loggerObj=log)

      dataObj = Utils.jsonLoadsSafe(jsonString=jsonString, default=jsonString, log=None)
      if log is not None:
        log.debug("dataObj: %s, type = %s", str(dataObj), str(type(dataObj)))

      # property has JSON structure
      rawSiteHeadersList = dataObj if isinstance(dataObj, list) else [dataObj]
      for rawSiteHeaders in rawSiteHeadersList:
        if HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME in rawSiteHeaders and rawSiteHeaders[HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME] != "" and \
          HTTPHeadersExtractor.PROPERTY_HEADERS_NAME in rawSiteHeaders and isinstance(url, basestring):
          if re.search(pattern=rawSiteHeaders[HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME], string=url, flags=re.I + re.U) is not None:
            headersObj = rawSiteHeaders[HTTPHeadersExtractor.PROPERTY_HEADERS_NAME]

            if HTTPHeadersExtractor.PROPERTY_STORAGE_NAME in rawSiteHeaders and rawSiteHeaders[HTTPHeadersExtractor.PROPERTY_STORAGE_NAME] != "":
              kwarg[HTTPHeadersExtractor.ARGUMENT_STORAGE_DIR] = rawSiteHeaders[HTTPHeadersExtractor.PROPERTY_STORAGE_NAME]

            if log is not None:
              log.debug("Domain '%s' passed for url: %s", str(rawSiteHeaders[HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME]), str(url))
              log.debug("headersObj: %s, type = %s", str(headersObj), str(type(headersObj)))

            if isinstance(headersObj, basestring):
              for elem in [h.strip() for h in headersObj.splitlines() if len(h) > 0 and h[0] != HTTPHeadersExtractor.DISABLE_CHARACTER]:
                elemList = elem.split(':')
                if len(elemList) > 1:
                  siteHeaders[elemList[0]] = elemList[1]
            elif isinstance(headersObj, dict):
              siteHeaders, saved = HTTPHeadersExtractor.updateSiteHeaders(siteHeaders=headersObj, url=url, log=log, **kwarg)

            break
          else:
            if log is not None:
              log.debug("Domain '%s' not passed for url: %s", str(rawSiteHeaders[HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME]), str(url))

#       if log is not None:
#         log.debug("dataObj2: %s, type = %s", str(dataObj), str(type(dataObj)))
#
#       if isinstance(dataObj, basestring):
#         for elem in [h.strip() for h in dataObj.splitlines() if len(h) > 0 and h[0] != HTTPHeadersExtractor.DISABLE_CHARACTER]:
#           elemList = elem.split(':')
#           if len(elemList) > 1:
#             siteHeaders[elemList[0]] = elemList[1]
#       elif isinstance(dataObj, dict):
#         siteHeaders, saved = HTTPHeadersExtractor.updateSiteHeaders(siteHeaders=dataObj, url=url, log=log, **kwarg)

      if log is not None:
        log.debug("siteHeaders: %s", str(siteHeaders))

    except Exception, err:
      if log is not None:
        log.error(str(err))

    return siteHeaders, saved


  # # load stored element from storage
  #
  # @param log - logger instance
  # @param kwarg - named arguments
  # @return stored element from storage
  @staticmethod
  def loadElement(log=None, **kwarg):
    #variable for result
    nodeElem = None
    try:
      siteId = HTTPHeadersExtractor.DEFAULT_SITEID_VALUE
      if HTTPHeadersExtractor.ARGUMENT_SITEID in kwarg:
        siteId = kwarg[HTTPHeadersExtractor.ARGUMENT_SITEID]

      domain = None
      if HTTPHeadersExtractor.ARGUMENT_DOMAIN in kwarg:
        domain = kwarg[HTTPHeadersExtractor.ARGUMENT_DOMAIN]

      storageDir = None
      if HTTPHeadersExtractor.ARGUMENT_STORAGE_DIR in kwarg:
        storageDir = kwarg[HTTPHeadersExtractor.ARGUMENT_STORAGE_DIR]

      dataStorage = LFSDataStorage()
      # load from storage
      nodeElem = dataStorage.loadElement(storageDir, domain, siteId)
      if log is not None:
        log.debug("nodeElem: %s, type = %s", str(nodeElem), str(type(nodeElem)))

    except Exception, err:
      if log is not None:
        log.error(str(err))

    return nodeElem


  # # save to storage
  #
  # @param element - data to save
  # @param log - logger instance
  # @param kwarg - named arguments
  # @return stored element from storage
  @staticmethod
  def saveElement(element, log=None, **kwarg):
    try:
      if isinstance(element, dict) and len(element) > 0:
        siteId = HTTPHeadersExtractor.DEFAULT_SITEID_VALUE
        if HTTPHeadersExtractor.ARGUMENT_SITEID in kwarg:
          siteId = kwarg[HTTPHeadersExtractor.ARGUMENT_SITEID]

        domain = None
        if HTTPHeadersExtractor.ARGUMENT_DOMAIN in kwarg:
          domain = kwarg[HTTPHeadersExtractor.ARGUMENT_DOMAIN]

        storageDir = None
        if HTTPHeadersExtractor.ARGUMENT_STORAGE_DIR in kwarg:
          storageDir = kwarg[HTTPHeadersExtractor.ARGUMENT_STORAGE_DIR]

        if log is not None:
          log.debug("Input element: %s, type = %s", str(element), str(type(element)))

        dataStorage = LFSDataStorage()
#         # load from storage
#         nodeElem = dataStorage.loadElement(storageDir, domain, siteId)
#         if log is not None:
#           log.debug("Loaded nodeElem: %s, type = %s", str(nodeElem), str(type(nodeElem)))
#         # merge data
#         if isinstance(nodeElem, dict) and isinstance(element, dict):
#           resDict = {}
#           for elemName in nodeElem.keys():
#             if elemName in element:
#               if log is not None:
#                 log.debug("elemName: %s", str(elemName))
#
#               resDict[elemName] = {}
#               for elementKey, elementValue in element[elemName].items():
#                 if log is not None:
#                   log.debug("elementKey: %s, elementValue: %s, nodeElem[elemName][elementKey]: %s", str(elementKey), str(elementValue), str(nodeElem[elemName][elementKey]))
#
#                 resDict[elemName][elementKey] = max(int(elementValue), int(nodeElem[elemName][elementKey]))
#
#           if log is not None:
#             log.debug("!!! resDict: %s", str(resDict))
#
#           element = resDict[elemName][elementKey]
#
        # save element to storage
        dataStorage.saveElement(storageDir, domain, siteId, element)
        if log is not None:
          log.debug("Save element: %s, type = %s", str(element), str(type(element)))

    except Exception, err:
      if log is not None:
        log.error(str(err))


  # # Get value by algorithm
  #
  # @param siteHeaders - site headers dictionary
  # @param url - url string
  # @param log - logger instance
  # @param kwarg - named arguments
  # @return site headers dictionary
  @staticmethod
  def updateSiteHeaders(siteHeaders, url=None, log=None, ** kwarg):
     # variable for result
    ret = {}
    saved = False

    if log is not None:
      log.debug("kwarg: %s", str(kwarg))
      log.debug("siteHeaders: %s, type = %s", str(siteHeaders), str(type(siteHeaders)))

    try:
      if isinstance(siteHeaders, dict):
        # set algorithm value
        algorithm = HTTPHeadersExtractor.DEFAULT_ALGORITHM_VALUE
        if HTTPHeadersExtractor.ARGUMENT_STORAGE_DIR in kwarg and kwarg[HTTPHeadersExtractor.ARGUMENT_STORAGE_DIR] is not None and \
          kwarg[HTTPHeadersExtractor.ARGUMENT_STORAGE_DIR] != "":
          algorithm = HTTPHeadersExtractor.ALGORITHM_LOW_FREQUENCY
          if HTTPHeadersExtractor.ARGUMENT_ALGORITHM in kwarg:
            algorithm = kwarg[HTTPHeadersExtractor.ARGUMENT_ALGORITHM]

#         nodeElem = None
        # initialization element by storage if necessary
        if algorithm == HTTPHeadersExtractor.ALGORITHM_LOW_FREQUENCY:
          # load element from storage
          nodeElem = HTTPHeadersExtractor.loadElement(log, **kwarg)
          # set default value
          if not isinstance(nodeElem, dict):
            nodeElem = {}

        # main loop for input site headers values
        for name, values in siteHeaders.items():
          if isinstance(values, basestring):
            if log is not None:
              log.debug("!!! name: %s, values: %s", str(name), str(values))

            if values.startswith(HTTPHeadersExtractor.FILE_SOURCE_PREFIX):
              resDict, saved = HTTPHeadersExtractor.getSiteHeaders(properties=values, url=url, log=log, **kwarg)
              if log is not None:
                log.debug("resDict: %s", str(resDict))
                log.debug("saved = %s", str(saved))

              for key, value in resDict.items():
                ret[key] = value

            else:
              ret[name] = values

          elif isinstance(values, list):
            # use different algorithms
            if algorithm == HTTPHeadersExtractor.ALGORITHM_RANDOM:
              # use random algorithm
              ret[name] = values[randint(0, len(values) - 1)]
            elif algorithm == HTTPHeadersExtractor.ALGORITHM_LOW_FREQUENCY:
              # use low frequency algorithm
              index = 0
              frequency = 0
              # set default value
              if name not in nodeElem or len(nodeElem[name]) != len(values) or "" in values:
                nodeElem[name] = {}
                while "" in values:
                  values.remove("")

                for i in range(0, len(values)):
                  nodeElem[name].update({str(i):frequency})

              if log is not None:
                log.debug("nodeElem1: %s", Utils.varDump(nodeElem, stringifyType=0))

              if isinstance(nodeElem, dict) and  name in nodeElem and isinstance(nodeElem[name], dict):
                if log is not None:
                  log.debug("nodeElem['%s']: %s, type = %s", str(name), str(nodeElem[name]), str(type(nodeElem)))

                for key, value in sorted(nodeElem[name].iteritems(), key=lambda (k, v): (v, k)):
                  if log is not None:
                    log.debug("key: %s, value: %s", str(key), str(value))

                for _index, _frequency in sorted(nodeElem[name].iteritems(), key=lambda (k, v): (v, k)):
                  index = int(_index)
                  frequency = int(_frequency)
                  break

              if log is not None:
                log.debug("index = %s, frequency = %s", str(index), str(frequency))

              if index < len(values):
                ret[name] = values[index]
                nodeElem[name].update({str(index):frequency + 1})

              if log is not None:
                log.debug("nodeElem: %s, type = %s", str(nodeElem), str(type(nodeElem)))

          elif isinstance(values, dict):
            if log is not None:
              log.debug("values: %s", str(values))

            headersData = None
            if '' in values:
              headersData = values['']
            else:
              hostName = Utils.getCurrentHostName()
              if log is not None:
                log.debug("Current host name: %s", str(hostName))

              if hostName in values:
                headersData = values[hostName]
              else:
                ip = Utils.getIP()
                if log is not None:
                  log.debug("Current host IP: %s", str(ip))

                if ip in values:
                  headersData = values[ip]

            if log is not None:
              log.debug("!!! headersData: %s", str(headersData))

            if headersData is None:
              break

            resDict, saved = HTTPHeadersExtractor.updateSiteHeaders({name:headersData}, url, log, **kwarg)
            if log is not None:
              log.debug("!!! resDict: %s", str(resDict))

            if name in resDict:
              ret[name] = resDict[name]
            else:
              ret[name] = headersData

        # save element to storage if necessary
        if algorithm == HTTPHeadersExtractor.ALGORITHM_LOW_FREQUENCY and not saved:
          HTTPHeadersExtractor.saveElement(nodeElem, log, **kwarg)
          saved = True

        if len(ret) == 0:
          if HTTPHeadersExtractor.PROPERTY_HEADERS_NAME in siteHeaders:
            ret = siteHeaders[HTTPHeadersExtractor.PROPERTY_HEADERS_NAME]
          else:
            ret = siteHeaders

    except Exception, err:
      if log is not None:
        log.error(str(err))

    if log is not None:
      log.debug("!!! ret: %s", str(ret))

    return ret, saved

#   # # extract HTTP headers
#   #
#   # @param siteProperties - site properties dictionary
#   # @param url - url string
#   # @param headers - headers dictionary as default value
#   # @param log - logger instance
#   # @return extracted HTTP headers
#   @staticmethod
#   def extract(siteProperties, url, headers=None, log=None):
#     # variable for result
#     siteHeaders = headers if isinstance(headers, dict) else {}
#     try:
#       if APP_CONSTS.HTTP_HEADERS in siteProperties and siteProperties[APP_CONSTS.HTTP_HEADERS] != "":
#         if log is not None:
#           log.debug("Found property: '%s': %s", str(APP_CONSTS.HTTP_HEADERS), str(siteProperties[APP_CONSTS.HTTP_HEADERS]))
#
#         lh = siteProperties[APP_CONSTS.HTTP_HEADERS]
#         if isinstance(siteProperties[APP_CONSTS.HTTP_HEADERS], basestring) and \
#           siteProperties[APP_CONSTS.HTTP_HEADERS].startswith(HTTPHeadersExtractor.FILE_SOURCE_PREFIX):
#           lh = Utils.loadFromFileByReference(fileReference=siteProperties[APP_CONSTS.HTTP_HEADERS], loggerObj=log)
#
#           if log is not None:
#             log.debug("lh: %s", str(lh))
#
#           for elem in [h.strip() for h in lh.splitlines() if len(h) > 0 and h[0] != HTTPHeadersExtractor.DISABLE_CHARACTER]:
#             elemList = elem.split(':')
#             if len(elemList) > 1:
#               siteHeaders[elemList[0]] = elemList[1]
#
#         else:
#           # property has JSON structure
#           rawSiteHeadersObj = Utils.jsonLoadsSafe(jsonString=siteProperties[APP_CONSTS.HTTP_HEADERS], default={}, log=log)
#
#           rawSiteHeadersList = rawSiteHeadersObj if isinstance(rawSiteHeadersObj, list) else [rawSiteHeadersObj]
#           for rawSiteHeaders in rawSiteHeadersList:
#             if HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME in rawSiteHeaders and rawSiteHeaders[HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME] != "" and \
#               HTTPHeadersExtractor.PROPERTY_HEADERS_NAME in rawSiteHeaders:
#               if re.search(pattern=rawSiteHeaders[HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME], string=url, flags=re.I + re.U) is not None:
#                 siteHeaders = rawSiteHeaders[HTTPHeadersExtractor.PROPERTY_HEADERS_NAME]
#                 if log is not None:
#                   log.debug("Domain '%s' passed for url: %s", str(rawSiteHeaders[HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME]), str(url))
#                 break
#               else:
#                 if log is not None:
#                   log.debug("Domain '%s' not passed for url: %s", str(rawSiteHeaders[HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME]), str(url))
#
#             else:
#               siteHeaders = rawSiteHeaders
#
#           if log is not None:
#             log.debug("siteHeaders: %s", Utils.varDump(siteHeaders))
#
#           for lh in siteHeaders:
#             if log is not None:
#               log.debug("['%s']  = '%s'", str(lh), str(siteHeaders[lh]))
#
#             headersData = None
#             if isinstance(siteHeaders[lh], list):
#               headersData = siteHeaders[lh][randint(0, len(siteHeaders[lh]) - 1)]
#             elif isinstance(siteHeaders[lh], basestring):
#               headersData = siteHeaders[lh]
#             elif isinstance(siteHeaders[lh], dict):
#               if '' in siteHeaders[lh]:
#                 headersData = siteHeaders[lh]['']
#               else:
#                 hostName = Utils.getCurrentHostName()
#                 if log is not None:
#                   log.debug("hostName: %s", str(hostName))
#
#                 if hostName in siteHeaders[lh]:
#                   headersData = siteHeaders[lh][hostName]
#                 else:
#                   ip = Utils.getIP()
#                   if log is not None:
#                     log.debug("IP: %s", str(ip))
#
#                   if ip in siteHeaders[lh]:
#                     headersData = siteHeaders[lh][ip]
#
#               if log is not None:
#                 log.debug("!!! headersData: %s", str(headersData))
#
#               if isinstance(headersData, list):
#                 siteHeaders[lh] = headersData[randint(0, len(headersData) - 1)]
#               elif isinstance(headersData, basestring):
#                 if headersData.startswith(HTTPHeadersExtractor.FILE_SOURCE_PREFIX):
#                   siteHeaders[lh] = Utils.loadFromFileByReference(fileReference=lh, loggerObj=log)
#                 else:
#                   siteHeaders[lh] = headersData
#
#           if log is not None:
#             log.info("len(siteHeaders) =  %s", str(len(siteHeaders)))
# #               log.info("siteHeaders: %s", Utils.varDump(siteHeaders))
#
#     except Exception, err:
#       if log is not None:
#         log.error(str(err))
#         log.info(Utils.getTracebackInfo())
#
#     return siteHeaders


  # # extract HTTP headers
  #
  # @param siteProperties - site properties dictionary
  # @param url - url string
  # @param headers - headers dictionary as default value
  # @param log - logger instance
  # @param kwarg - named arguments
  # @return extracted HTTP headers
  @staticmethod
  def extract(siteProperties, url, headers=None, log=None, **kwarg):
    # variable for result
    siteHeaders = headers if isinstance(headers, dict) else {}
    try:
      if APP_CONSTS.HTTP_HEADERS in siteProperties and siteProperties[APP_CONSTS.HTTP_HEADERS] != "":
        if log is not None:
          log.debug("Found property: '%s': %s", str(APP_CONSTS.HTTP_HEADERS), str(siteProperties[APP_CONSTS.HTTP_HEADERS]))

        propertiesValue = siteProperties[APP_CONSTS.HTTP_HEADERS]
        extractedSiteHeaders, saved = HTTPHeadersExtractor.getSiteHeaders(properties=propertiesValue, url=url, log=log, **kwarg)  # pylint: disable=W0612
        if isinstance(extractedSiteHeaders, dict) and len(extractedSiteHeaders) > 0:
          siteHeaders = extractedSiteHeaders

    except Exception, err:
      if log is not None:
        log.error(str(err))
        log.info(Utils.getTracebackInfo())

    if log is not None:
      log.debug("!!! Result siteHeaders: %s', type = %s", str(siteHeaders), str(type(siteHeaders)))

    return siteHeaders

