# coding: utf-8
"""
HCE project,  Python bindings, Distributed Tasks Manager application.
HTTPProxy Class content main functional for transformation proxy name to dictionary.

@package: dc_crawler
@file HTTPProxy.py
@author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2018 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""

import re


class HTTPProxy(object):
  # # Constants used in class
  PROXY_PROTOCOL_HTTP = 'http'
  PROXY_PROTOCOL_HTTPS = 'https'

  PROXY_PROTOCOL_SOCKS_PREFIX = 'socks'

  DEFAULT_PROXY_PROTOCOL = PROXY_PROTOCOL_HTTP
  DEFAULT_PROXY_PROTOCOL_LIST = [PROXY_PROTOCOL_HTTP, PROXY_PROTOCOL_HTTPS]
  DEFAULT_PORT_HTTP = 80
  DEFAULT_PORT_HTTPS = 443
  DEFAULT_PORT_SOCKS = 1080


  # Initialization
  def __init__(self):
    pass


  # # Get proxy tuple
  #
  # @param proxyName - proxy name
  # @param log - logger instance
  # @param defaultProxyProtocol - default proxy protocol
  # @return proxy tuple or None
  @staticmethod
  def getProxyTuple(proxyName, log=None, defaultProxyProtocol=DEFAULT_PROXY_PROTOCOL):
    # variables for result
    ret = None
    if log is not None:
      log.debug("proxyName: %s", str(proxyName))

    if isinstance(proxyName, basestring) and proxyName != "":
      proxyType = proxyHost = proxyPort = proxyUser = proxyPassword = None
      try:
        pattern = '(.*)://(.*):(.*)@(.*):(.*)'
        match = re.search(pattern, proxyName, re.I + re.U)
        if match is not None:
          proxyType, proxyUser, proxyPassword, proxyHost, proxyPort = match.groups()

        else:
          pattern = '(.*)://(.*):(.*)'
          match = re.search(pattern, proxyName, re.I + re.U)
          if match is not None:
            proxyType, proxyHost, proxyPort = match.groups()
          else:
            pattern = '(.*)://(.*)'
            match = re.search(pattern, proxyName, re.I + re.U)
            if match is not None:
              proxyType, proxyHost = match.groups()
            else:
              pattern = '(.*):(.*)'
              match = re.search(pattern, proxyName, re.I + re.U)
              if match is not None:
                proxyHost, proxyPort = match.groups()
                proxyType = defaultProxyProtocol

        # initialization of the port by default values
        if proxyPort is None:
          if proxyType == HTTPProxy.PROXY_PROTOCOL_HTTP:
            proxyPort = HTTPProxy.DEFAULT_PORT_HTTP
          elif proxyType == HTTPProxy.PROXY_PROTOCOL_HTTPS:
            proxyPort = HTTPProxy.DEFAULT_PORT_HTTPS
          elif re.search(pattern=HTTPProxy.PROXY_PROTOCOL_SOCKS_PREFIX, string=proxyType, flags=re.I + re.U) is not None:
            proxyPort = HTTPProxy.DEFAULT_PORT_SOCKS

        ret = (proxyType, proxyHost, proxyPort, proxyUser, proxyPassword)
      except Exception, err:
        if log is not None:
          log.error(str(err))

    return ret


  # Get proxy string
  #
  # @param type - proxy type
  # @param host - proxy host
  # @param port - proxy port
  # @param user - proxy user
  # @param password - proxy password
  # @param tuple - tuple of proxy
  # @return proxy string
  @staticmethod
  def getProxyString(**kwargs):
    # variable for result
    ret = None

    if 'tuple' in kwargs and kwargs['tuple'] is not None:
      pType, host, port, user, password = kwargs['tuple']
      ret = HTTPProxy.getProxyString(type=HTTPProxy.DEFAULT_PROXY_PROTOCOL if pType is None else pType, host=host, port=port, user=user, password=password)

    elif 'type' in kwargs and kwargs['type'] is not None and \
      'host' in kwargs and kwargs['host'] is not None and \
      'port' in kwargs and kwargs['port'] is not None and \
      'user' in kwargs and kwargs['user'] is not None and \
      'password' in kwargs and kwargs['password'] is not None:
      ret = "%s://%s:%s@%s:%s" % (kwargs['type'], kwargs['user'], kwargs['password'], kwargs['host'], kwargs['port'])

    elif 'type' in kwargs and kwargs['type'] is not None and \
      'host' in kwargs and kwargs['host'] is not None and \
      'port' in kwargs and kwargs['port'] is not None:
      ret = "%s://%s:%s" % (kwargs['type'], kwargs['host'], kwargs['port'])

    return ret


  # # Get proxies dictionary
  #
  # @param proxyTuple - proxy tuple
  # @param log - logger instance
  # @return proxies dictionary
  @staticmethod
  def getProxiesDict(proxyTuple, log=None):
    # variable for result
    ret = None
    try:
#       if log is not None:
#         log.debug("proxyTuple: %s", str(proxyTuple))

      if proxyTuple is not None:
        pType, host, port, user, password = proxyTuple
        proxies = {}
        for proxyType in HTTPProxy.DEFAULT_PROXY_PROTOCOL_LIST:
          proxyString = HTTPProxy.getProxyString(type=HTTPProxy.DEFAULT_PROXY_PROTOCOL if pType is None else pType,
                                                 host=host,
                                                 port=port,
                                                 user=user,
                                                 password=password)
#           if log is not None:
#             log.debug("proxyType = '%s', proxyString: '%s'", str(proxyType), str(proxyString))

          if proxyString is not None:
            proxies[proxyType] = proxyString

        if len(proxies) > 0:
          ret = proxies

#         if log is not None:
#           log.debug("ret: %s", str(ret))

    except Exception, err:
      if log is not None:
        log.error(str(err))

    return ret


  # # Get proxies dictionary
  #
  # @param proxyName - proxy name
  # @param log - logger instance
  # @return proxies dictionary
  @staticmethod
  def getProxies(proxyName, log=None):
    return HTTPProxy.getProxiesDict(proxyTuple=HTTPProxy.getProxyTuple(proxyName=proxyName, log=log, defaultProxyProtocol=None), log=log)
