# coding: utf-8
"""
HCE project,  Python bindings, Distributed Tasks Manager application.
HTTPRedirectHeadersDebug Class content main functional for 'HTTP_REDIRECT_HEADERS_DEBUG' property.

@package: dc_crawler
@file HTTPRedirectHeadersDebug.py
@author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2019 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""

import app.Consts as APP_CONSTS
import app.Utils as Utils


class HTTPRedirectHeadersDebug(object):
  # # Constants used in class
  DEBUG_OFF = 0
  DEBUG_ON = 1
  
  TEMPLATE_REDIRECT_URL= 'X-Redirect-%N%'
  TEMPLATE_STATUS_CODE= 'X-StatusCode-%N%'
  MACRO_NUMBER = '%N%'
  
  # # Initialization
  def __init__(self):
    pass


  # # Is active state
  #
  # @param siteProperties - site properties dictionary
  # @param log - logger instance
  # @return True if state is active or False otherwise
  @staticmethod
  def isActive(siteProperties=None, log=None):
    # variable for result
    state = HTTPRedirectHeadersDebug.DEBUG_OFF
    try:
      sitePropertiesDict = Utils.jsonLoadsSafe(jsonString=siteProperties, default=siteProperties, log=None)
      if isinstance(sitePropertiesDict, dict) and  APP_CONSTS.HTTP_REDIRECT_HEADERS_DEBUG in sitePropertiesDict:
        if isinstance(sitePropertiesDict[APP_CONSTS.HTTP_REDIRECT_HEADERS_DEBUG], int) and \
          sitePropertiesDict[APP_CONSTS.HTTP_REDIRECT_HEADERS_DEBUG] >= HTTPRedirectHeadersDebug.DEBUG_ON:
          state = HTTPRedirectHeadersDebug.DEBUG_ON
        elif isinstance(sitePropertiesDict[APP_CONSTS.HTTP_REDIRECT_HEADERS_DEBUG], basestring) and \
          sitePropertiesDict[APP_CONSTS.HTTP_REDIRECT_HEADERS_DEBUG].isdigit() and \
          int(sitePropertiesDict[APP_CONSTS.HTTP_REDIRECT_HEADERS_DEBUG]) >= HTTPRedirectHeadersDebug.DEBUG_ON:
          state = HTTPRedirectHeadersDebug.DEBUG_ON
    except Exception, err:
      if log is not None:
        log.error(str(err))

    return state == HTTPRedirectHeadersDebug.DEBUG_ON


  # # Add data
  #
  # @param redirectUrl - redirect URL
  # @param statusCode - status code
  # @param siteProperties - site properties dictionary
  # @param log - logger instance
  # @param results - external results list of a dictionaries elements
  # @return results list of a dictionaries elements
  @staticmethod
  def addData(redirectUrl, statusCode, siteProperties, log, results):
    if HTTPRedirectHeadersDebug.isActive(siteProperties, log) and isinstance(results, list):
      currentNumber = str(len(results))
      dataDict = {}
      dataDict[HTTPRedirectHeadersDebug.TEMPLATE_REDIRECT_URL.replace(HTTPRedirectHeadersDebug.MACRO_NUMBER, currentNumber)] = str(redirectUrl)
      dataDict[HTTPRedirectHeadersDebug.TEMPLATE_STATUS_CODE.replace(HTTPRedirectHeadersDebug.MACRO_NUMBER, currentNumber)] = str(statusCode)
      results.append(dataDict)

    return results


  # # Update response headers
  #
  # @param newHeaderElements - list of a dictionaries new headers elements
  # @param implRes - response instance
  # @return updated response instance
  @staticmethod
  def updateResponseHeaders(newHeaderElements, implRes):
    if isinstance(newHeaderElements, list) and implRes is not None:
      for elem in newHeaderElements:
        if isinstance(elem, dict):
          for key, value in elem.items():
            implRes.headers[key] = value

    return implRes
