# coding: utf-8

"""
HCE project, Python bindings, Distributed Tasks Manager application.
HostAliveExtractor it's class content main functional for properties 'HOST_ALIVE_CHECK_NAME' and 'HOST_ALIVE_CHECK_PROXY_NAME'.

@package: dc_crawler
@file HostAliveExtractor.py
@author bgv, Alexander Vybornyh <alexander.hce.cluster@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2018 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""

import urlparse
import app.Utils as Utils
import app.Consts as APP_CONSTS


class HostAliveExtractor(object):
  # #Constants used in class
  DEFAULT_PROTOCOL_PREFIX = 'http://'

  PARAMETER_METHOD = 'method'
  PARAMETER_DOMAIN_NAME_RESOLVE = 'domain_name_resolve'
  PARAMETER_CONNECT_RESOLVE = 'connect_resolve'
  PARAMETER_CONNECTION_TIMEOUT = 'connection_timeout'


  # # Initialization
  def __init__(self):
    pass


  # # Check available of host
  #
  # @param url - input url for check
  # @param parameters - input parameters dict
  # @param log - logger instance
  # @param timeout - timeout value
  # @return boolean flag True if success, otherwise False
  @staticmethod
  def isHostAvailable(url, parameters, log=None, timeout=0.5):
    # variable for result
    ret = True

    if log is not None:
      log.debug("url: %s, parameters: %s", str(url), str(parameters))
    try:
      if not isinstance(url, basestring) or url == "":
        raise Exception("Bad parameter 'url'")

      if HostAliveExtractor.PARAMETER_METHOD in parameters and int(parameters[HostAliveExtractor.PARAMETER_METHOD]) == 0:
        # from urlparse import urlparse
        pr = urlparse.urlparse(url)
        # print str(pr)
        pr = pr.netloc.split(':')
        if len(pr) == 1:
          port = 80
        else:
          port = int(pr[1])
        host = pr[0]
        # print host, port
        if HostAliveExtractor.PARAMETER_DOMAIN_NAME_RESOLVE in parameters and int(parameters[HostAliveExtractor.PARAMETER_DOMAIN_NAME_RESOLVE]) == 1:
          import socket
          ai = socket.getaddrinfo(host, port, 0, 0, socket.IPPROTO_TCP)
          # print ai
          if HostAliveExtractor.PARAMETER_CONNECT_RESOLVE in parameters and int(parameters[HostAliveExtractor.PARAMETER_CONNECT_RESOLVE]) == 1:
            if HostAliveExtractor.PARAMETER_CONNECTION_TIMEOUT in parameters and float(parameters[HostAliveExtractor.PARAMETER_CONNECTION_TIMEOUT]) > 0:
              timeout = float(parameters[HostAliveExtractor.PARAMETER_CONNECTION_TIMEOUT])
            for item in ai:
              af, socktype, proto, canonname, sa = item  # pylint: disable=W0612
              s = socket.socket(af, socktype, proto)
              s.settimeout(float(timeout))
              try:
                s.connect(sa)
              except Exception, err:
                ret = False
                # print str(sa), str(err)
                if log is not None:
                  log.debug("Host %s, timeout %f connect check error: %s", str(sa), str(timeout), str(err))
                continue
              s.close()
              ret = True
              break

    except Exception, err:
      ret = False
      if log is not None:
        log.debug("Host %s availability check error: %s", str(url), str(err))
        log.info(Utils.getTracebackInfo())

    return ret


  # # host alive property common handler
  #
  # @param propertyName - property name
  # @param siteProperties - site properties
  # @param url - input url for check
  # @param log - logger instance
  @staticmethod
  def hostAliveHandler(propertyName, siteProperties, url, log=None):
    # variable for result
    ret = True
    if propertyName in siteProperties:
      try:
        if log is not None:
          log.debug("Property '%s' found in site properties", str(propertyName))

        parameters = Utils.jsonLoadsSafe(jsonString=siteProperties[propertyName], default=siteProperties[propertyName], log=log)
        if log is not None:
          log.debug("Property '%s' successfully got from json", str(propertyName))

        ret = HostAliveExtractor.isHostAvailable(url, parameters, log)
      except Exception, err:
        if log is not None:
          log.error("Try getting '%s' was fail: %s", str(propertyName), str(err))

    return ret


  # # check is available url or not
  #
  # @param siteProperties - site properties
  # @param url - input url for check
  # @param log - logger instance
  # @return True - if available or False otherwise
  @staticmethod
  def isAvailableUrl(siteProperties, url, log=None):
    return HostAliveExtractor.hostAliveHandler(propertyName=APP_CONSTS.HOST_ALIVE_CHECK,
                                               siteProperties=siteProperties,
                                               url=url,
                                               log=log)


  # # check is available proxy or not
  #
  # @param siteProperties - site properties
  # @param proxyName - input proxy name for check
  # @param log - logger instance
  # @return True - if available or False otherwise
  @staticmethod
  def isAvailableProxy(siteProperties, proxyName, log=None):
    return HostAliveExtractor.hostAliveHandler(propertyName=APP_CONSTS.HOST_ALIVE_CHECK_PROXY,
                                               siteProperties=siteProperties,
                                               url=HostAliveExtractor.DEFAULT_PROTOCOL_PREFIX + proxyName,
                                               log=log)
