"""
HCE project,  Python bindings, Distributed Tasks Manager application.
SiteReprocessTask class content main functional of support site re-process.

@package: dc_db
@file SiteReprocessTask.py
@author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2018 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""

import os
import sys

import dc.EventObjects as EventObjects
from dc_db.BaseTask import BaseTask
import dc_db.Constants as Constants
from dtm.EventObjects import GeneralResponse
import app.Utils as Utils

logger = Utils.MPLogger().getLogger()


# # class implemented all logic necessary to process SiteReprocess request
#
class SiteReprocessTask(BaseTask):
  # # Constants used in class
  TEMPLATE_REPROCESS_SQL = "UPDATE `urls_%s` SET `Status`='%s' WHERE `ParentMd5` <> '' AND `Status` IN (%s)"
  # UPDATE `urls_0` SET `Status`='4' WHERE `ParentMd5` <> '' AND `Status` IN ('5', '6', '7')

  # # Constants used in class
  DEFAULT_VALUE_STATUS = EventObjects.URL.STATUS_CRAWLED
  DEFAULT_VALUE_OLD_STATUSES = [EventObjects.URL.STATUS_SELECTED_PROCESSING, EventObjects.URL.STATUS_PROCESSING, EventObjects.URL.STATUS_PROCESSED]


  # # Initialization
  def __init__(self):
    super(SiteReprocessTask, self).__init__()
    self.errorCode = 0
    self.errorMessage = "OK"


  # # make all necessary actions to re-process site data
  #
  # @param siteReprocesses - list of instances of the SiteReprocess objects
  # @param queryCallback function for queries execution
  # @return generalResponse instance of GeneralResponse object
  def process(self, siteReprocesses, queryCallback):
    ret = GeneralResponse()
    if not isinstance(siteReprocesses, list):
      siteReprocesses = [siteReprocesses]

    for siteReprocess in siteReprocesses:
      self.errorCode = 0
      self.errorMessage = "OK"
      if self.isSiteExist(siteReprocess.id, queryCallback):

        logger.info("Re-Process for %s started", str(siteReprocess.id))

        self.updateUrlsStatuses(siteReprocess, queryCallback)

        logger.info("Re-Process for %s finished", str(siteReprocess.id))
      else:
        self.errorCode = Constants.EXIT_CODE_GLOBAL_ERROR
        self.errorMessage = ("Site id [%s] not found" % siteReprocess.id)

      ret.errorCode = self.errorCode
      ret.statuses.append(ret.errorCode)
      if ret.errorMessage is None or ret.errorMessage == "":
        ret.errorMessage = self.errorMessage
      else:
        ret.errorMessage += ("-" + self.errorMessage)

    return ret

  # # update url's status for re-process execution
  #
  # @param siteReprocess - SiteReprocess instance
  # @param queryCallback function for queries execution
  # @return - None
  def updateUrlsStatuses(self, siteReprocess, queryCallback):
    query = self.makeUpdateStatusQuery(siteReprocess)
    logger.info("!!! query: %s", str(query))
    if query is not None:
      queryCallback(query, Constants.SECONDARY_DB_ID)


  # # make update status query
  #
  # @param siteReprocess - SiteReprocess instance
  # @return query string
  def makeUpdateStatusQuery(self, siteReprocess):
    #variable for result
    ret = None
    try:
      status = self.DEFAULT_VALUE_STATUS if siteReprocess.status is None else int(siteReprocess.status)
      oldStatuses = self.DEFAULT_VALUE_OLD_STATUSES if siteReprocess.oldStatuses is None else siteReprocess.oldStatuses
  
      if not isinstance(oldStatuses, list):
        oldStatuses = [oldStatuses]
        
      criterionIn = ""
      for oldStatus in oldStatuses:
        criterionIn += "'" + str(oldStatus) + "',"

      criterionIn = criterionIn.rstrip(',') if criterionIn.endswith(',') else criterionIn

      ret = self.TEMPLATE_REPROCESS_SQL % (str(siteReprocess.id), str(status), criterionIn)

    except Exception, err:
      self.errorCode = Constants.EXIT_CODE_GLOBAL_ERROR
      self.errorMessage = str(err)
      logger.error(self.errorMessage)

    return ret


  # #sets empty values in `sites` table for some fields
  #
  # @param siteCleanup instance of SiteCleanup object
  # @param queryCallback function for quieries execution
  def cleanUpMysqlSiteTable(self, siteCleanup, queryCallback):
    CLEAR_SITE_RECORS_SQL = ("UPDATE `sites` SET TcDate=NOW(), Resources=0, Iterations=0, State=%s, " +
                             "ErrorMask=0, Errors=0, Contents=0, CollectedURLs=0, TotalURLs=0, TotalContents=0 WHERE id = '%s'")
    query = CLEAR_SITE_RECORS_SQL % (str(siteCleanup.state), siteCleanup.id)
    queryCallback(query, Constants.PRIMARY_DB_ID)

    tbName = Constants.DC_URLS_TABLE_NAME_TEMPLATE % siteCleanup.id
    # UPDATE `urls_0` SET `Status`='4' WHERE `ParentMd5` <> '' AND `Status` > '4'

